AutoMidHandle(True)
Type TLoadedImage 'Diese Klasse verwaltet die Bilder die in den Speicher geladen werden
	Global lAllImages:TList = New TList
	
	'Eigenschaften des Bildes
	Field tiImage:TImage
	Field sPath:String
	Field sName:String
	Field iWidth:Int, iHeight:Int
	Field bAnimated:Byte
		
	'Folgende nur fr animierte Bilder
	Field iCellWidth:Int = 0, iCellHeight:Int = 0
	Field iFramesInTotal:Int = 0
	Field iTimer:Int = 0
	
	Method New()
		lAllImages.AddLast(Self) 'Bild wird automatisch der Liste hinzugefgt
	End Method
	
	Method Destroy()
		lAllImages.Remove(Self) 'Der Destruktor lscht das Bild automatisch aus der Liste
	End Method
	
	Function Create:TLoadedImage(sPath:String, sName:String) 'Konstruktor fr ein nicht animiertes Bild
		'* berprfen ob das Bild nicht schon vorhanden ist *
		For Local Img:TLoadedImage = EachIn lAllImages
			If (Img.sName = sName) Then
				If (Img.sPath = sPath) Then
					Return Null
				EndIf
			EndIf
		Next
		
		Local Image:TLoadedImage = New TLoadedImage
		
		Image.sPath = sPath
		Image.sName = sName
		Image.bAnimated = False
		Image.tiImage = LoadImage(sPath)
		Image.iWidth = ImageWidth(Image.tiImage)
		Image.iheight = ImageHeight(Image.tiImage)
		
		Return Image
	End Function
	
	'*Konstruktor fr ein animiertes Bild*
	Function CreateAnimated:TLoadedImage(sPath:String, sName:String, iCellWidth:Int, iCellHeight:Int, iFramesInTotal:Int, iTimer:Int)
		For Local Img:TLoadedImage = EachIn lAllImages
			If (Img.sName = sName) Then
				If (Img.sPath = sPath) Then
					Return Null
				EndIf
			EndIf
		Next
		
		Local Image:TLoadedImage = New TLoadedImage
		
		Image.sPath = sPath
		Image.sName = sName
		Image.bAnimated = True
		Image.tiImage = LoadAnimImage(sPath, iCellwidth, iCellheight, 0, iFramesInTotal)
		Image.iWidth = ImageWidth(Image.tiImage)
		Image.iheight = ImageHeight(Image.tiImage)
		
		Image.iCellwidth = iCellwidth
		Image.iCellheight = iCellheight
		Image.iFramesInTotal = iFramesInTotal
		Image.iTimer = iTimer
		
		Return Image
	End Function
	
	Function GetImage:TLoadedImage(sName:String) 'Ein bestimmtes Bild anhand des Namens zurckgeben
	'Diese Function knnte noch genauer suchen wenn auch nach dem Pfad gesucht wird, jedoch kann das Problem von doppelten Namen durch
	'Den Programmierer umgangen werden
		For Local img:TLoadedImage = EachIn lAllImages
			If (Lower(img.sname) = Lower(sName)) Then
				Return img
			EndIf
		Next
		Return Null
	End Function
End Type

Type TDrawnImage 'Diese Klasse stellt ein Bild fr jedes Objekt dar. Also: 100 Gegner = 10 Instanzen dieser Klasse
	'Eigenschaften des Bildes
	Field tiImage:TImage
	Field iWidth:Int, iHeight:Int
	Field bAnimated:Byte
		
	'Folgende nur fr animierte Bilder
	Field iFramesInTotal:Int = 0
	Field iActiveFrame:Int = 0
	Field iTimer:Int = 0
	Field iLastUpDate:Int = 0
	
	Function Create:TDrawnImage(img:TLoadedImage)
		Local Image:TDrawnImage = New TDrawnImage
			
		Image.tiImage = img.tiImage
		Image.bAnimated = img.bAnimated
		
		If (img.bAnimated) Then
			Image.iWidth = img.iCellwidth
			Image.iHeight = img.iCellHeight
			Image.iFramesInTotal = img.iFramesInTotal
			Image.iTimer = img.iTimer
			Image.iActiveFrame = 0
			Image.iLastUpDate = MilliSecs()
		Else
			Image.iWidth = img.iWidth
			Image.iHeight = img.iHeight
		EndIf
			
		Return Image
	End Function
	
	Method Animate() 'Das Bild wird automatisch animiert (Zeitsteuerung)
		If (bAnimated)
			If ((iTimer + iLastUpDate) < MilliSecs()) Then
				iActiveFrame = (iActiveFrame + 1) Mod iFramesInTotal
				iLastUpDate = MilliSecs()
			EndIf
		EndIf
	End Method
	
	Method AnimateManual() 'Das Bild wird einmal einen Animationsschritt weiter gebracht
		If (bAnimated)
			iActiveFrame = (iActiveFrame + 1) Mod iFramesInTotal
		EndIf
	End Method

	Method Draw(v2dPos:TVector2D) 'Das Bild wird mit der momentanen Animation gemalt
		DrawImage(tiImage, v2dPos.Get_X(), v2dPos.Get_Y(), iActiveFrame)
	End Method

End Type
