Type TVector2D 'A Vector Class which can do any Operation on 2D Vectors
	Field _fXComponent:Float 'X-Component of the Vector
	Field _fYComponent:Float 'Y-Component of the Vector
	Field _fAbsoluteValue:Float 'The Absoulte Value (Length) of the Vector
	
	Method Get_X:Float() 'Method to get the X-Component
		Return Self._fXComponent
	End Method
	Method Set_X(px:Float) 'Method to set the X-Component
		Self._fXComponent = px
		Self._fAbsoluteValue = Self.CalcLength()
	End Method
	
	Method Get_Y:Float() 'Method to get the Y-Component
		Return Self._fYComponent
	End Method
	Method Set_Y(py:Float) 'Method to set the Y-Component
		Self._fYComponent = py
		Self._fAbsoluteValue = Self.CalcLength()
	End Method
	
	Method Get_Length:Float() 'Method to get the Absolute Value (Length)
		Return Self._fAbsoluteValue
	End Method
	
	Method PrintValues()
		Print "X-Component: " + Self._fXComponent
		Print "Y-Component: " + Self._fYComponent
		Print "Absolute Value: " + Self._fAbsoluteValue
	End Method
	
	Method GetValuesAsString:String()
		Local tmpString:String = ("X/Y: " + Self._fXComponent + " / " + Self._fYComponent + " |vec|: " + Self._fAbsoluteValue)
		Return tmpString
	End Method
	
	'Constructor of the Class
	Function Create:TVector2D(pX:Float, pY:Float)
		Local tmpVec2D:TVector2D = New TVector2D
		tmpVec2D._fXComponent = pX
		tmpVec2D._fYComponent = pY
		tmpVec2D._fAbsoluteValue = tmpVec2D.CalcLength()
		Return tmpVec2D
	End Function
	
	'"Constructor" for a Zero Vector
	Function Zero:TVector2D()
		Local tmpVec2D:TVector2D = New TVector2D
		tmpVec2D = TVector2D.Create(0.0, 0.0) 'Just calls the Constructor with the parameters 0, 0
		Return tmpVec2D
	End Function
	
	'Calculates the Length of a Vector
	Method CalcLength:Float()
		Return Sqr((Self._fXComponent * Self._fXComponent) + (Self._fYComponent * Self._fYComponent))
	End Method
	
	 'Calculates the distance between two Vectors
	Function CalcDistance:Float(pVec1:TVector2D, pVec2:TVector2D)
		Return Sqr(((pVec1._fXComponent - pVec2._fXComponent) * (pvec1._fXComponent - pvec2._fXComponent)) + ((pvec1._fYComponent - pvec2._fYComponent) * (pvec1._fYComponent - pvec2._fYComponent)))
	End Function
	
	'Add two Vectors
	Function Add:TVector2D(pVec1:TVector2D, pVec2:TVector2D)
		Local tmpVec:TVector2D = New TVector2D
		tmpVec = TVector2D.Create(pVec1._fXComponent + pVec2._fXComponent, pVec1._fYComponent + pVec2._fYComponent)
		Return tmpVec
	End Function
	
	'Subtract two Vectors (vec1-vec2)
	Function Subtract:TVector2D(pVec1:TVector2D, pVec2:TVector2D) 
		Local tmpVec:TVector2D = New TVector2D
		tmpVec = TVector2D.Create(pVec1._fXComponent - pVec2._fXComponent, pVec1._fYComponent - pVec2._fYComponent)
		Return tmpVec
	End Function
	
	'Multiplies a Vector with a number (makes the Vector shorter or longer)
	Method Multiply:TVector2D(pFactor:Float) 
		Local tmpVec:TVector2D = New TVector2D
		tmpVec = TVector2D.Create(pFactor * Self._fXComponent, pFactor * Self._fYComponent)
		Return tmpVec
	End Method
	
	'Calculates the Scalar Product of two Vectors
	Function ScalarProduct:Float(pVec1:TVector2D, pVec2:TVector2D) 
		Return ((pVec1._fXComponent * pVec2._fXComponent) + (pVec1._fYComponent * pVec2._fYComponent))
	End Function
	
	'Calculates the angle between two Vectors
	Function AngleBetween:Int(pVec1:TVector2D, pVec2:TVector2D)
		Local tmpResult:Float
		'tmpResult =
		tmpResult = ScalarProduct(pVec1, pVec2) / (pVec1._fAbsoluteValue * pVec2._fAbsoluteValue)
		If ((tmpResult <= 1) And (tmpResult >= - 1)) Then 'Need to be a number between -1 and 1
			Return Floor(ACos(tmpResult) + 0.5)
		Else
			Return (-1) 'Else it will be -1 (because the result of acos is between 180 and 0)
		EndIf
	End Function
	
	'Tests if two Vectors are orthogonal (right angled)
	Function AreOrthogonal:Int(pVec1:TVector2D, pVec2:TVector2D) 
		Local tmpValue:Float
		tmpValue = ScalarProduct(pVec1, pVec2)
		If tmpValue = 0 Then
			Return True
		Else
			Return False
		EndIf
	End Function
	
	'Calculates the orthogonal projection of Vec1 on Vec2
	Function OrthogonalProjection:TVector2D(pVec1:TVector2D, pVec2:TVector2D)
		Local tmpValue:Float
		Local tmpVec:TVector2D = New TVector2D
		tmpVec = TVector2D.Zero()
		tmpValue = ScalarProduct(pVec1, pVec2.Standardise())
		tmpVec = pVec2.Standardise().Multiply(tmpValue)
		tmpVec.CalcLength()
		Return tmpVec
	End Function
	
	'Calculates the unit Vector
	Method Standardise:TVector2D()
		Local tmpVec:TVector2D = New TVector2D
		Local tmpValue:Float
		tmpValue = 1 / Self._fAbsoluteValue
		tmpVec = Self.Multiply(tmpValue)
		Return tmpVec
	End Method
End Type